/*
 *  CPE 474 lab 0 - modern graphics test bed
 *  draws a partial cube using a VBO and IBO 
 *  glut/OpenGL/GLSL application   
 *  Uses glm and local matrix stack
 *  to handle matrix transforms for a view matrix, projection matrix and
 *  model transform matrix
 *
 *  zwood 9/12 
 *  Copyright 2012 Cal Poly. All rights reserved.
 *
 *****************************************************************************/
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif
#ifdef __unix__
#include <GL/glut.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
//#include <jpeglib.h>
//#include <jerror.h>
#include "finalProg.h"
/*#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr
#include "GLSL_helper.h"
#include "MStackHelp.h"
#include "GeometryCreator.h"
#include "CMeshLoaderSimple.h"
#include "TextureLoader.h"
#include "ParticleSystem.h"*/
//#include "noise.h"

using namespace glm;
using namespace std;


/* Base function was used to calculate the y position of all vertexes in the
 * water mesh. I updated this function to calculate the vertexes more to a 
 * way that fit my simulation. I added support for multiple ripple points
 * in the water and took the average.*/
static float	getHeight (const float x, const float y, const float t)
{
  float x2 = x;
  float y2 = y;
  float xx = x2 * x2;
  float yy = y2 * y2;
  float pt1 =  ((-sinf (2 * sqrtf (xx + yy) - 4 * -t))/10);
  x2 = x + 20;
  y2 = y + 30;
  xx = x2*x2;
  y2 = y2*y2;
  float pt2 = ((-sinf (2 * sqrtf (xx + yy) - 4 * -t))/10);
  x2 = x - 30;
  y2 = y - 10;
  xx = x2*x2;
  y2 = y2*y2;
  float pt3 = ((-sinf (2 * sqrtf (xx + yy) - 4 * -t))/10);
  
  //return pt1 * pt2;
  return (pt1 + pt2 + pt3)/3;

}


//forward declaration of image loading and texture set-up code
//int ImageLoad(char *filename, Image *image);
//GLvoid LoadTexture(char* image_file, int tex_id);


void SetMaterial(int i) {

  //glUseProgram(ShadeProg);
  switch (i) {
    case 0:
        safe_glUniform3f(h_uMatAmb, 0.4, 0.2, 0.2);
        safe_glUniform3f(h_uMatDif, 0.6, 0.4, 0.4);
        safe_glUniform3f(h_uMatSpec, 0.4, 0.3, 0.3);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 1:
        safe_glUniform3f(h_uMatAmb, 0.1, 0.1, 0.2);
        safe_glUniform3f(h_uMatDif, 0.3, 0.3, 0.4);
        safe_glUniform3f(h_uMatSpec, 0.3, 0.3, 0.4);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 2:
        safe_glUniform3f(h_uMatAmb, 0.1, 0.4, 0.1);
        safe_glUniform3f(h_uMatDif, 0.4, 0.6, 0.4);
        safe_glUniform3f(h_uMatSpec, 0.3, 0.4, 0.3);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 4:
        safe_glUniform3f(h_uMatAmb, 0.1, 0.4, 0.1);
        safe_glUniform3f(h_uMatDif, 0.4, 0.6, 0.4);
        safe_glUniform3f(h_uMatSpec, 0.3, 0.4, 0.3);
        safe_glUniform1f(h_uMatShine, 200.0);
        break;
    case 3:
        safe_glUniform3f(h_uMatAmb, 0.2, 0.3, 0.5);
        safe_glUniform3f(h_uMatDif, 0, 0.4, 0.9);
        safe_glUniform3f(h_uMatSpec, 0.4, 0.4, 0.4);
        safe_glUniform1f(h_uMatShine, 200.0);
        break;
    case 5:
        safe_glUniform3f(h_uMatAmb, 0.62, 0.72, 0.77);
        safe_glUniform3f(h_uMatDif, 0.62, 0.72, 0.77);
        safe_glUniform3f(h_uMatSpec, 0.62, 0.72, 0.77);
        safe_glUniform1f(h_uMatShine, 200.0);
        break;
    case 6:
        safe_glUniform3f(h_uMatAmb, 0.9, 0.0, 0.0);
        safe_glUniform3f(h_uMatDif, 0.9, 0.0, 0.0);
        safe_glUniform3f(h_uMatSpec, 0.9, 0.0, 0.0);
        safe_glUniform1f(h_uMatShine, 200.0);
        break;
    case 7:
        safe_glUniform3f(h_uMatAmb, 0, 0.0, 0.0);
        safe_glUniform3f(h_uMatDif, 0, 0.0, 0.0);
        safe_glUniform3f(h_uMatSpec, 0, 0.0, 0.0);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
    case 10:
        safe_glUniform3f(h_uMatAmb, .75, .75, .75);
        safe_glUniform3f(h_uMatDif, .75, .75, .75);
        safe_glUniform3f(h_uMatSpec, .75, .75, .75);
        safe_glUniform1f(h_uMatShine, 1.0);
        break;
  }
}

void SetTexMaterial(int i) {

  //glUseProgram(ShadeProg);
  switch (i) {
    case 0:
        //safe_glUniform3f(h_tuMatAmb, 0.5, 0.3, 0.3);
        //safe_glUniform3f(h_tuMatDif, 0.6, 0.4, 0.4);
        //safe_glUniform3f(h_tuMatSpec, 0.4, 0.3, 0.3);
        safe_glUniform1f(h_tuMatShine, 1.0);
        safe_glUniform1i(h_tuLeaf, 0);
        break;
    case 1:
        //safe_glUniform3f(h_tuMatAmb, 0.3, 0.4, 0.6);
        //safe_glUniform3f(h_tuMatDif, 0, 0.4, 0.9);
        //safe_glUniform3f(h_tuMatSpec, 0.4, 0.4, 0.4);
        safe_glUniform1f(h_tuMatShine, 1.0);
        safe_glUniform1i(h_tuLeaf, 0);
        break;
    case 2:
        //safe_glUniform3f(h_tuMatAmb, 0.3, 0.4, 0.6);
        //safe_glUniform3f(h_tuMatDif, 0, 0.4, 0.9);
        //safe_glUniform3f(h_tuMatSpec, 0.4, 0.4, 0.4);
        safe_glUniform1i(h_tuLeaf, 1);
        safe_glUniform1f(h_tuMatShine, 1.0);
        break;
    case 3:
        //safe_glUniform3f(h_tuMatAmb, 0.3, 0.4, 0.6);
        //safe_glUniform3f(h_tuMatDif, 0, 0.4, 0.9);
        //safe_glUniform3f(h_tuMatSpec, 0.4, 0.4, 0.4);
        safe_glUniform1f(h_tuMatShine, 1.0);
        safe_glUniform1i(h_tuLeaf, 0);
        break;
  }
}
/* projection matrix - do not change - sets matrix in shader */
void SetProjectionMatrix() {
  glm::mat4 Projection = glm::perspective(80.0f, (float)g_width/g_height, 0.1f, 120.f);	
  safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}

void SetTexProjectionMatrix() {
  glm::mat4 Projection = glm::perspective(80.0f, (float)g_width/g_height, 0.1f, 120.f);	
  safe_glUniformMatrix4fv(h_tuProjMatrix, glm::value_ptr(Projection));
}

void SetView() {
   glm::mat4 View = glm::lookAt(eyeVec, lookAtVec, upVec); 
//   glm::mat4 View = glm::lookAt(glm::vec3(0, 0, 0), glm::vec3(vx, vy, vz), glm::vec3(0, 1, 0)); 
   safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(View));
}

void SetTexView() {
   glm::mat4 View = glm::lookAt(eyeVec, lookAtVec, upVec); 
//   glm::mat4 View = glm::lookAt(glm::vec3(0, 0, 0), glm::vec3(vx, vy, vz), glm::vec3(0, 1, 0)); 
   safe_glUniformMatrix4fv(h_tuViewMatrix, glm::value_ptr(View));
}


/* camera controls - do not change - sets matrix in shader */
/*void SetView() {
  glm::mat4 Trans = glm::translate( glm::mat4(1.0f), glm::vec3(0.0f, 0, g_trans));
  glm::mat4 RotateX = glm::rotate( Trans, g_angle, glm::vec3(0.0f, 1, 0));
  safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(RotateX));
}*/

/* model transforms - do not change - sets matrix in shader */
void SetModel() {
  //if(!isTextured) { 
   safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(ModelTrans.modelViewMatrix));
  //}
  //else {
  //}
}

void SetTexModel() {
  //if(!isTextured) { 
  //}
  //else {
   safe_glUniformMatrix4fv(h_tuModelMatrix, glm::value_ptr(ModelTrans.modelViewMatrix));
  //}
}
/* intialize ground data */
/*static void initGround() {
   water = GeometryCreator::CreateCube(vec3(g_groundSize*2, 0.2, g_groundSize*2));
}*/


//static void initBoatShadow() {


static void initSky() {
   //skyCube = GeometryCreator::CreateCube(vec3(80, 80, 80));
   skyCube = GeometryCreator::CreateInvertSphere();
   float skyTexts[] = {
      0, 0, // back 
      0, 1,
      0.25, 1,
      0.25, 0,
      0.75, 0, //front 
      0.75, 1,
      0.5, 1,
      0.5, 0,
      0.75, 0, //left
      1, 0,
      1, 1,
      0.75, 1,
      0.5, 0, // right 
      0.25, 0,
      0.25, 1,
      0.5, 1,
      0.9, 0.9,
      0.9, 1,
      1, 1,
      1, 0.9,
      0.9, 0.9,
      0.9, 1,
      1, 1,
      1, 0.9
  };
    static float const CubeNormals[] =
    {
         0,  0, 1, // back face verts [0-3]
         0,  0, 1,
         0,  0, 1,
         0,  0, 1,
         
         0,  0,  -1, // front face verts [4-7]
         0,  0,  -1,
         0,  0,  -1,
         0,  0,  -1,
         
        1,  0,  0, // left face verts [8-11]
        1,  0,  0,
        1,  0,  0,
        1,  0,  0,
        
         -1,  0,  0, // right face verts [12-15]
         -1,  0,  0,
         -1,  0,  0,
         -1,  0,  0,
        
         0,  -1,  0, // top face verts [16-19]
         0,  -1,  0,
         0,  -1,  0,
         0,  -1,  0,
        
         0, 1,  0, // bottom face verts [20-23]
         0, 1,  0,
         0, 1,  0,
         0, 1,  0
    };

    /*glGenBuffers(1, &SkyNormObj);
    glBindBuffer(GL_ARRAY_BUFFER, SkyNormObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubeNormals), CubeNormals, GL_STATIC_DRAW);
    
    glGenBuffers(1, &STexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, STexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(skyTexts), skyTexts, GL_STATIC_DRAW);*/ 

}

static void initBoat() {
   boatBase = GeometryCreator::CreateCube(vec3(1, 0.05, 2.5));
   boatStern = GeometryCreator::CreateCube(vec3(1, 0.5, 0.05));
   boatBow = GeometryCreator::CreateCube(vec3(1, 0.5, 0.05));
   boatSides = GeometryCreator::CreateCube(vec3(0.05, 0.5, 2));
   //boatMast = GeometryCreator::CreateCube(vec3(0.2, 4, 0.2));
   //boatSail = GeometryCreator::CreateCube(vec3(0.1, 2, 3));
   
   float boatTexts[] = {
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0};

    
    glGenBuffers(1, &TexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(boatTexts), boatTexts, GL_STATIC_DRAW);

}

static void initLand() {
   land = GeometryCreator::CreateSphere(vec3(10.0f, 0.5f, 10.0f), 128, 32); 
   
}

/* Most of the algorithm from this function was gotten
 * from tutorial water code. The base code calculated the water
 * vertexes and normals for the array of triangle strips used. It initially
 * created a triangle map of a fixed length. I changed this to support a variable
 * width and depth of the water. I also replaced a lot of the equations with
 * glm functions for finding cross products, etc.
 */
void initWater() {

  float t = 0;
  int texIndice;

  for (j = 0; j < RESOLUTION; j++)
  {
      z = WATER_WIDTH_DEPTH*(j + 1) * delta - WATER_WIDTH_DEPTH;
      for (i = 0; i <= RESOLUTION; i++)
	{
	  indice = 6 * (i + j * (RESOLUTION + 1));
	  x =  WATER_WIDTH_DEPTH*i * delta - WATER_WIDTH_DEPTH;
	  surface[indice + 3] = x;
	  surface[indice + 4] = getHeight(x, z, t);
	  surface[indice + 5] = z;
          if (j != 0)
	    {
	      /* Values were computed during the previous loop */
	      preindice = 6 * (i + (j - 1) * (RESOLUTION + 1));
	      surface[indice] = surface[preindice + 3];
	      surface[indice + 1] = surface[preindice + 4];
	      surface[indice + 2] = surface[preindice + 5];
	    }
	  else
	    {
	      surface[indice] = x;
	      surface[indice + 1] = getHeight(x, -WATER_WIDTH_DEPTH, t);
	      surface[indice + 2] = -WATER_WIDTH_DEPTH;
	    }
	}
  }
  for (j = 0; j < RESOLUTION; j++) {
    for (i = 0; i <= RESOLUTION; i++)
      {
	indice = 6 * (i + j * (RESOLUTION + 1));
        texIndice = 4 *(i + j * (RESOLUTION + 1));

	v1x = surface[indice + 3];
	v1y = surface[indice + 4];
	v1z = surface[indice + 5];

	v2x = v1x;
	v2y = surface[indice + 1];
	v2z = surface[indice + 2];

	if (i < RESOLUTION)
	  {
	    v3x = surface[indice + 9];
	    v3y = surface[indice + 10];
	    v3z = v1z;
	  }
	else
	  {
	    v3x = xn;
	    v3y = getHeight(xn, v1z, t);
	    v3z = v1z;
	  }

         /*vax = v2x - v1x;
         vay = v2y - v1y;
         vaz = v2z - v1z;
         vbx = v3x - v1x;
         vby = v3y - v1y;
         vbz = v3z - v1z;*/

         tmpVec1.x = v2x - v1x;
         tmpVec1.y = v2y - v1y;
         tmpVec1.z = v2z - v1z;
         tmpVec2.x = v3x - v1x;
         tmpVec2.y = v3y - v1y;
         tmpVec2.z = v3z - v1z;
         norm = glm::cross(tmpVec2, tmpVec1);

         nx = norm.x;
         ny = norm.y;
         nz = norm.z;

        /*nx = (vby * vaz) - (vbz * vay);
	ny = (vbz * vax) - (vbx * vaz);
	nz = (vbx * vay) - (vby * vax);*/

	l = glm::length(norm);	if (l != 0)
	  {
	    l = 1 / l;
	    normal[indice + 3] = nx * l;
            texts[texIndice + 2] = nx * l;
	    normal[indice + 4] = ny * l;
            texts[texIndice + 3] = ny * l;
	    normal[indice + 5] = nz * l;
	  }
	else
	  {
	    normal[indice + 3] = 0;
            texts[texIndice + 2] = 0;
	    normal[indice + 4] = 1;
            texts[texIndice + 3] = 1;
	    normal[indice + 5] = 0;
	  }


	if (j != 0)
	  {
	    /* Values were computed during the previous loop*/ 
	    preindice = 6 * (i + (j - 1) * (RESOLUTION + 1));
	    texts[texIndice] = normal[indice] = normal[preindice + 3];
	    texts[texIndice + 1] = normal[indice + 1] = normal[preindice + 4];
	    normal[indice + 2] = normal[preindice + 5];
	  }
	else
	  {
	    v1y = getHeight(v1x, (j - 1) * delta - 1, t);
	    v1z = (j - 1) * delta - 1;

	    v3y = getHeight(v3x, v2z, t);
	    v3z = v2z;

	    /*vax = v1x - v2x;
	    vay = v1y - v2y;
	    vaz = v1z - v2z;

	    vbx = v3x - v2x;
	    vby = v3y - v2y;
	    vbz = v3z - v2z;

	    nx = (vby * vaz) - (vbz * vay);
	    ny = (vbz * vax) - (vbx * vaz);
	    nz = (vbx * vay) - (vby * vax);*/

            tmpVec1.x = v1x - v2x;
            tmpVec1.y = v1y - v2y;
            tmpVec1.z = v1z - v2z;
            tmpVec2.x = v3x - v2x;
            tmpVec2.y = v3y - v2y;
            tmpVec2.z = v3z - v2z;
            norm = glm::cross(tmpVec2, tmpVec1);

            nx = norm.x;
            ny = norm.y;
            nz = norm.z;


	    l = glm::length(norm);//sqrtf (nx * nx + ny * ny + nz * nz);
	    if (l != 0)
	      {
		l = 1 / l;
		texts[texIndice] = normal[indice] = nx * l;
		texts[texIndice + 1] = normal[indice + 1] = ny * l;
		normal[indice + 2] = nz * l;
	      }
	    else
	      {
		texts[texIndice] = normal[indice] = 0;
		texts[texIndice + 1] = normal[indice + 1] = 1;
		normal[indice + 2] = 0;
	      }
	  }
      }
   }


    glGenBuffers(1, &WTexBuffObj); 
    glBindBuffer(GL_ARRAY_BUFFER, WTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(texts), texts, GL_STATIC_DRAW);

    glGenBuffers(1, &waterBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, waterBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(surface), surface, GL_STATIC_DRAW);

    glGenBuffers(1, & waterNormObj);
    glBindBuffer(GL_ARRAY_BUFFER, waterNormObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(normal), normal, GL_STATIC_DRAW);
}

void initPalm() { 
   palmBase = GeometryCreator::CreateCube(vec3(0.2, 6, 0.2));
   palmLeaf = GeometryCreator::CreateCube(vec3(4, 0.1, 1));

   float PalmTexts[] = {
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,1,
      1,1,
      1,0};

   float PalmLeaves[] = {
      0,0,
      0,.1,
      .1,.1,
      .1,0,
      0,0,
      0,.1,
      .1,.1,
      .1,0,
      0,0,
      0,.1,
      .1,.1,
      .1,0,
      0,0,
      0,.1,
      .1,.1,
      .1,0,
      0,0,
      0,1,
      1,1,
      1,0,
      0,0,
      0,.1,
      .1,.1,
      .1,0};
    
    glGenBuffers(1, &PTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, PTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(PalmTexts), PalmTexts, GL_STATIC_DRAW);

    glGenBuffers(1, &PLTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(PalmLeaves), PalmLeaves, GL_STATIC_DRAW);
}

void initParticles() {

  particle = GeometryCreator::CreateSphere(glm::vec3(0.15, 0.05, 0.2));

  // Fill HouseKeeping Array of Particle positions
  allParticles.initalize();
}

void InitGeom() { 
  CMeshLoader::loadVertexBufferObjectFromMesh("Models/gameguy1k.m", TriangleCount, triBuffObj, colBuffObj, normalBuffObj);
  personShadow = GeometryCreator::CreateSphere(vec3(1, 0, 1), 128, 32);
  initLand();
  initBoat();
  initWater();
  initSky();
  initPalm();
  initParticles();
}


int InstallTexShader(const GLchar *vShaderName, const GLchar *fShaderName) {
        GLuint VS; //handles to shader object
        GLuint FS; //handles to frag shader object
        GLint vCompiled, fCompiled, linked; //status of shader

        VS = glCreateShader(GL_VERTEX_SHADER);
        FS = glCreateShader(GL_FRAGMENT_SHADER);

        //load the source
        glShaderSource(VS, 1, &vShaderName, NULL);
        glShaderSource(FS, 1, &fShaderName, NULL);

        //compile shader and print log
        glCompileShader(VS);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
        printShaderInfoLog(VS);

        //compile shader and print log
        glCompileShader(FS);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
        printShaderInfoLog(FS);

        if (!vCompiled || !fCompiled) {
                printf("Error compiling either shader %s or %s", vShaderName, fShaderName);
                return 0;
        }

        //create a program object and attach the compiled shader
        TexShadeProg = glCreateProgram();
        glAttachShader(TexShadeProg, VS);
        glAttachShader(TexShadeProg, FS);

        glLinkProgram(TexShadeProg);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetProgramiv(TexShadeProg, GL_LINK_STATUS, &linked);
        printProgramInfoLog(TexShadeProg);

        glUseProgram(TexShadeProg);

        /* get handles to attribute data */
       h_taPosition = safe_glGetAttribLocation(TexShadeProg, "aPosition");
       h_taNormal = safe_glGetAttribLocation(TexShadeProg,	"aNormal");
       h_taTexCoord = safe_glGetAttribLocation(TexShadeProg,  "aTexCoord");
       h_tuTexUnit = safe_glGetUniformLocation(TexShadeProg, "uTexUnit");
       h_tuProjMatrix = safe_glGetUniformLocation(TexShadeProg, "uProjMatrix");
       h_tuViewMatrix = safe_glGetUniformLocation(TexShadeProg, "uViewMatrix");
       h_tuModelMatrix = safe_glGetUniformLocation(TexShadeProg, "uModelMatrix");
       //h_tuWaterToggle = safe_glGetUniformLocation(TexShadeProg, "water");
       

     h_tuLightPos = safe_glGetUniformLocation(TexShadeProg, "uLightPos");
     h_tuLightColor = safe_glGetUniformLocation(TexShadeProg, "uLColor");
     //h_tuMatAmb = safe_glGetUniformLocation(TexShadeProg, "uMat.aColor");
     //h_tuMatDif = safe_glGetUniformLocation(TexShadeProg, "uMat.dColor");
     //h_tuMatSpec = safe_glGetUniformLocation(TexShadeProg, "uMat.sColor");
     h_tuMatShine = safe_glGetUniformLocation(TexShadeProg, "uMat.shine");
     h_tuView = safe_glGetUniformLocation(TexShadeProg, "uView"); 
     h_tuLeaf = safe_glGetUniformLocation(TexShadeProg, "leaf"); 
       printf("sucessfully installed shader %d\n", TexShadeProg);
       return 1;

}

/*function to help load the shaders (both vertex and fragment */
int InstallShader(const GLchar *vShaderName, const GLchar *fShaderName) {
        GLuint VS; //handles to shader object
        GLuint FS; //handles to frag shader object
        GLint vCompiled, fCompiled, linked; //status of shader

        VS = glCreateShader(GL_VERTEX_SHADER);
        FS = glCreateShader(GL_FRAGMENT_SHADER);

        //load the source
        glShaderSource(VS, 1, &vShaderName, NULL);
        glShaderSource(FS, 1, &fShaderName, NULL);

        //compile shader and print log
        glCompileShader(VS);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
        printShaderInfoLog(VS);

        //compile shader and print log
        glCompileShader(FS);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
        printShaderInfoLog(FS);

        if (!vCompiled || !fCompiled) {
                printf("Error compiling either shader %s or %s", vShaderName, fShaderName);
                return 0;
        }

        //create a program object and attach the compiled shader
        ShadeProg = glCreateProgram();
        glAttachShader(ShadeProg, VS);
        glAttachShader(ShadeProg, FS);

        glLinkProgram(ShadeProg);
        /* check shader status requires helper functions */
        printOpenGLError();
        glGetProgramiv(ShadeProg, GL_LINK_STATUS, &linked);
        printProgramInfoLog(ShadeProg);

        glUseProgram(ShadeProg);

        /* get handles to attribute data */
        h_aPosition = safe_glGetAttribLocation(ShadeProg, "aPosition");
        /* add a handle for the normal */
	h_aNormal = safe_glGetAttribLocation(ShadeProg,	"aNormal");
        //h_uColor = safe_glGetUniformLocation(ShadeProg,  "uColor");
    	h_uProjMatrix = safe_glGetUniformLocation(ShadeProg, "uProjMatrix");
    	h_uViewMatrix = safe_glGetUniformLocation(ShadeProg, "uViewMatrix");
    	h_uModelMatrix = safe_glGetUniformLocation(ShadeProg, "uModelMatrix");
     h_uLightPos = safe_glGetUniformLocation(ShadeProg, "uLightPos");
     h_uLightColor = safe_glGetUniformLocation(ShadeProg, "uLColor");
     h_uMatAmb = safe_glGetUniformLocation(ShadeProg, "uMat.aColor");
     h_uMatDif = safe_glGetUniformLocation(ShadeProg, "uMat.dColor");
     h_uMatSpec = safe_glGetUniformLocation(ShadeProg, "uMat.sColor");
     h_uMatShine = safe_glGetUniformLocation(ShadeProg, "uMat.shine");
     h_uView = safe_glGetUniformLocation(ShadeProg, "uView"); 
       printf("sucessfully installed shader %d\n", ShadeProg);
       return 1;
}


/* Some OpenGL initialization */
void Initialize ()					// Any GL Init Code 
{
	// Start Of User Initialization
	glClearColor (1.0f, 1.0f, 1.0f, 1.0f);								
	// Black Background
 	glClearDepth (1.0f);	// Depth Buffer Setup
 	glDepthFunc (GL_LEQUAL);	// The Type Of Depth Testing
	glEnable (GL_DEPTH_TEST);// Enable Depth Testing

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_FRONT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    /* some matrix stack init */
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();
 
}

void DrawPalm(vec3 transl, int angle, vec3 scale) {

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE4);
    glBindTexture(GL_TEXTURE_2D, 4);
    safe_glUniform1i(h_tuTexUnit, 4);
   
    glDisable(GL_CULL_FACE);
   
   ModelTrans.pushMatrix();
      SetTexMaterial(0);
      ModelTrans.translate(transl);
      ModelTrans.scale(scale.x, scale.y, scale.z);
      ModelTrans.rotate(angle, vec3(0, 1, 0));
      SetTexModel();
      safe_glEnableVertexAttribArray(h_taPosition);
      glBindBuffer(GL_ARRAY_BUFFER, palmBase->PositionHandle);
      safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
      safe_glEnableVertexAttribArray(h_taNormal);
      glBindBuffer(GL_ARRAY_BUFFER, palmBase->NormalHandle);
      safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
      safe_glEnableVertexAttribArray(h_taTexCoord);
      glBindBuffer(GL_ARRAY_BUFFER, PTexBuffObj);
      safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
      glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmBase->IndexHandle);
      glDrawElements(GL_TRIANGLES, palmBase->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            glDisable(GL_TEXTURE_2D);
            glEnable(GL_TEXTURE_2D);
            glActiveTexture(GL_TEXTURE5);
            glBindTexture(GL_TEXTURE_2D, 5);
            safe_glUniform1i(h_tuTexUnit, 5);
            glEnable(GL_BLEND);
            glBlendFunc(GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA);
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               SetTexMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(-15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(94, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(-89, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(43, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(-40, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(137, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(-133, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(15, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(-120, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(-4, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(40, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(-5, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(117, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(-3, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
            ModelTrans.pushMatrix();
               ModelTrans.rotate(-30, vec3(0, 1, 0));
               ModelTrans.translate(vec3(0, 3, 0));
               ModelTrans.rotate(-6, vec3(0, 0, 1));
               ModelTrans.translate(vec3(-1.5, 0, 0));
               //SetMaterial(2);
               SetTexModel();
               safe_glEnableVertexAttribArray(h_taPosition);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->PositionHandle);
               safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taNormal);
               glBindBuffer(GL_ARRAY_BUFFER, palmLeaf->NormalHandle);
               safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
               safe_glEnableVertexAttribArray(h_taTexCoord);
               glBindBuffer(GL_ARRAY_BUFFER, PLTexBuffObj);
               safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
               glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, palmLeaf->IndexHandle);
               glDrawElements(GL_TRIANGLES, palmLeaf->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
            ModelTrans.popMatrix();
   ModelTrans.popMatrix();

   glEnable(GL_CULL_FACE);
   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
   safe_glDisableVertexAttribArray(h_aPosition);
   safe_glDisableVertexAttribArray(h_aNormal);
}

void DrawSoldier(vec3 transl, int angle, vec3 scale) {
   ModelTrans.pushMatrix();
      SetMaterial(3);
      ModelTrans.translate(transl);
      ModelTrans.scale(scale.x, scale.y, scale.z);
      ModelTrans.rotate(angle, vec3(0, 1,0));
      SetModel();
      safe_glEnableVertexAttribArray(h_aPosition);
      glBindBuffer(GL_ARRAY_BUFFER, triBuffObj);
      safe_glVertexAttribPointer(h_aPosition, 4, GL_FLOAT, GL_FALSE, 0, 0);
      safe_glEnableVertexAttribArray(h_aNormal);
      glBindBuffer(GL_ARRAY_BUFFER, normalBuffObj);
      safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
      //glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatSides->IndexHandle);
      //glUniform3f(h_uColor, 0.67, 0.41, 0.07);
      glDrawArrays(GL_TRIANGLES, 0, TriangleCount*3);
      //glDrawElements(GL_TRIANGLES, boatSides->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.pushMatrix();
         SetMaterial(7);
         ModelTrans.translate(vec3(-.1, -.5, .1));
         ModelTrans.scale(0.2, 0.2, 0.2);
         SetModel();
         safe_glEnableVertexAttribArray(h_aPosition);
         glBindBuffer(GL_ARRAY_BUFFER, personShadow->PositionHandle);
         safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_aNormal);
         glBindBuffer(GL_ARRAY_BUFFER, personShadow->NormalHandle);
         safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, personShadow->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         glDrawElements(GL_TRIANGLES, personShadow->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
   ModelTrans.popMatrix();
}

void DrawSky() {
    glCullFace(GL_FRONT);
    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE2);
    glBindTexture(GL_TEXTURE_2D, 2);
    safe_glUniform1i(h_tuTexUnit, 2);
    //printf("here!\n");
    SetTexMaterial(1);
    safe_glEnableVertexAttribArray(h_taTexCoord);
    //safe_glEnableVertexAttribArray(h_taNormal);
    //glBindBuffer(GL_ARRAY_BUFFER, STexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, skyCube->TextureHandle);
    safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(0, 20, 0));
    ModelTrans.rotate(150, vec3(0, 0, 1));
    //ModelTrans.scale(0.3, 0.3, 0.3);
    SetTexModel();
    safe_glEnableVertexAttribArray(h_taPosition);
    glBindBuffer(GL_ARRAY_BUFFER, skyCube->PositionHandle);
    safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    safe_glEnableVertexAttribArray(h_taNormal);
    //glBindBuffer(GL_ARRAY_BUFFER, SkyNormObj);
    glBindBuffer(GL_ARRAY_BUFFER, skyCube->NormalHandle);
    safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, skyCube->IndexHandle);
      glDrawElements(GL_TRIANGLES, skyCube->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    ModelTrans.popMatrix();
      safe_glDisableVertexAttribArray(h_taPosition);
      safe_glDisableVertexAttribArray(h_taNormal);
    glDisable(GL_TEXTURE_2D);
    glCullFace(GL_BACK);
}

void DrawParticles() {
   //printf("velocity: %f\n", velocity);
   if(velocity < -0.05) {
      //printf("drawing particles!\n");
      glDisable(GL_TEXTURE_2D);
      glEnable(GL_TEXTURE_2D);
      glActiveTexture(GL_TEXTURE1);
      glBindTexture(GL_TEXTURE_2D, 1);
      safe_glUniform1i(h_tuTexUnit, 1);
      for (int index = 0; index < NUM_PARTICLES; index++) {
         ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(allParticles.particles[index].position.x, allParticles.particles[index].position.y,
                                      allParticles.particles[index].position.z));
            SetTexMaterial(3);
            safe_glEnableVertexAttribArray(h_taPosition);
            glBindBuffer(GL_ARRAY_BUFFER, particle->PositionHandle);
            safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, particle->IndexHandle);
            safe_glEnableVertexAttribArray(h_taNormal);
            glBindBuffer(GL_ARRAY_BUFFER, particle->NormalHandle);
            safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
            safe_glEnableVertexAttribArray(h_taTexCoord);
            //safe_glEnableVertexAttribArray(h_taNormal);
            //glBindBuffer(GL_ARRAY_BUFFER, STexBuffObj);
            glBindBuffer(GL_ARRAY_BUFFER, particle->TextureHandle);
            safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
            SetTexModel();
            glDrawElements(GL_TRIANGLES, particle->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
         ModelTrans.popMatrix();
      }

       allParticles.update(0.01);

       glDisable(GL_TEXTURE_2D);
       glEnable(GL_TEXTURE_2D);
       glActiveTexture(GL_TEXTURE0);
       glBindTexture(GL_TEXTURE_2D, 0);
       safe_glUniform1i(h_tuTexUnit, 0);
   }

}

void DrawBoat(vec3 transl, int angle, vec3 scale) {

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, 0);
    safe_glUniform1i(h_tuTexUnit, 0);

    safe_glEnableVertexAttribArray(h_taTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
    safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
    
    SetTexMaterial(0);
    ModelTrans.pushMatrix();
      ModelTrans.translate(transl);
      ModelTrans.scale(scale.x, scale.y, scale.z);
      ModelTrans.rotate(angle, vec3(0, 1, 0));
      ModelTrans.translate(vec3(0, yPos, -2.1));
      ModelTrans.rotate(boyAng, vec3(0, 0, 1));
      ModelTrans.pushMatrix();
      ModelTrans.translate(vec3(0, 0, -0.5));
      SetTexModel();
      safe_glEnableVertexAttribArray(h_taPosition);
      glBindBuffer(GL_ARRAY_BUFFER, boatBase->PositionHandle);
      safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
      safe_glEnableVertexAttribArray(h_taNormal);
      glBindBuffer(GL_ARRAY_BUFFER, boatBase->NormalHandle);
      safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
      glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatBase->IndexHandle);
    safe_glEnableVertexAttribArray(h_taTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
    safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
      //glUniform3f(h_uColor, 0.67, 0.41, 0.07);
      //draw 
      glDrawElements(GL_TRIANGLES, boatBase->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
      DrawParticles();
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(-0.5, 0.25, 0));
         SetTexModel();
         safe_glEnableVertexAttribArray(h_taPosition);
         glBindBuffer(GL_ARRAY_BUFFER, boatSides->PositionHandle);
         safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_taNormal);
         glBindBuffer(GL_ARRAY_BUFFER, boatSides->NormalHandle);
         safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatSides->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         safe_glEnableVertexAttribArray(h_taTexCoord);
         glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
         safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
         // draw!
         glDrawElements(GL_TRIANGLES, boatSides->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(0.5, 0.25, 0));
         SetTexModel();
         safe_glEnableVertexAttribArray(h_taPosition);
         glBindBuffer(GL_ARRAY_BUFFER, boatSides->PositionHandle);
         safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_taNormal);
         glBindBuffer(GL_ARRAY_BUFFER, boatSides->NormalHandle);
         safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatSides->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         safe_glEnableVertexAttribArray(h_taTexCoord);
         glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
         safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
         // draw!
         glDrawElements(GL_TRIANGLES, boatSides->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(-0.25, 0.25, -1.25));
         ModelTrans.rotate(60, vec3(0, 1, 0));
         SetTexModel();
         safe_glEnableVertexAttribArray(h_taPosition);
         glBindBuffer(GL_ARRAY_BUFFER, boatBow->PositionHandle);
         safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_taNormal);
         glBindBuffer(GL_ARRAY_BUFFER, boatBow->NormalHandle);
         safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatBow->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         safe_glEnableVertexAttribArray(h_taTexCoord);
         glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
         safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
         // draw!
         glDrawElements(GL_TRIANGLES, boatBow->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(0.25, 0.25, -1.25));
         ModelTrans.rotate(-60, vec3(0, 1, 0));
         SetTexModel();
         safe_glEnableVertexAttribArray(h_taPosition);
         glBindBuffer(GL_ARRAY_BUFFER, boatBow->PositionHandle);
         safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_taNormal);
         glBindBuffer(GL_ARRAY_BUFFER, boatBow->NormalHandle);
         safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatBow->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         safe_glEnableVertexAttribArray(h_taTexCoord);
         glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
         safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
         // draw!
         glDrawElements(GL_TRIANGLES, boatBow->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
      ModelTrans.pushMatrix();
         ModelTrans.translate(vec3(0, 0.25, 1));
         SetTexModel();
         safe_glEnableVertexAttribArray(h_taPosition);
         glBindBuffer(GL_ARRAY_BUFFER, boatStern->PositionHandle);
         safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
         safe_glEnableVertexAttribArray(h_taNormal);
         glBindBuffer(GL_ARRAY_BUFFER, boatStern->NormalHandle);
         safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
         glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, boatStern->IndexHandle);
         glUniform3f(h_uColor, 0.67, 0.41, 0.07);
         safe_glEnableVertexAttribArray(h_taTexCoord);
         glBindBuffer(GL_ARRAY_BUFFER, TexBuffObj);
         safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 
         // draw!
         glDrawElements(GL_TRIANGLES, boatStern->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      ModelTrans.popMatrix();
   ModelTrans.popMatrix();

      safe_glDisableVertexAttribArray(h_taPosition);
      safe_glDisableVertexAttribArray(h_taNormal);
    glDisable(GL_TEXTURE_2D);
}


/* Similar to initWater, this function uses the same algorithm to calculate
 * vertex and normal values for the array of triangle strips that make the water
 * mesh. I updated this function to only update the y-portion of the vertexes in the
 * mesh. I also added in texture mapping to this algorithm that dynamically change when
 * drawn. Lastly, I added glm functionality for vector calculations.
 **/
void drawSea() {
  float t = (elapsedTime ++) /100;
  int texIndice;
  
  for (j = 0; j < RESOLUTION; j++) {
      //Get the current z coordinate of the triangle strip array.
      z =  WATER_WIDTH_DEPTH*(j + 1) * delta - WATER_WIDTH_DEPTH;

      for (i = 0; i <= RESOLUTION; i++)
	{
	  indice = 6 * (i + j * (RESOLUTION + 1));
          
          //Get the current x coordinate of the triangle strip array.
	  x = WATER_WIDTH_DEPTH*i * delta - WATER_WIDTH_DEPTH;
	  
          surface[indice + 4] = getHeight(x, z, t);
	  if (j != 0) {
	      /* These are created to take into account repeating vertices. Get the previous
               * row in the grid, with the same column, and get those values.*/
	      preindice = 6 * (i + (j - 1) * (RESOLUTION + 1));
	      surface[indice + 1] = surface[preindice + 4];
          }
	  else
	    {
	      //surface[indice] = x;
	      surface[indice + 1] = getHeight(x, -WATER_WIDTH_DEPTH, t);
	      //surface[indice + 2] = -20;
	    }
	}
  }

  for (j = 0; j < RESOLUTION; j++) {
    for (i = 0; i <= RESOLUTION; i++)
      {
	indice = 6 * (i + j * (RESOLUTION + 1));
        texIndice = 4 *(i + j * (RESOLUTION + 1));

	v1x = surface[indice + 3];
	v1y = surface[indice + 4];
	v1z = surface[indice + 5];

	v2x = v1x;
	v2y = surface[indice + 1];
	v2z = surface[indice + 2];

	if (i < RESOLUTION)
	  {
	    v3x = surface[indice + 9];
	    v3y = surface[indice + 10];
	    v3z = v1z;
	  }
	else
	  {
	    v3x = xn;
	    v3y = getHeight(xn, v1z, t);
	    v3z = v1z;
	  }

         /*vax = v2x - v1x;
         vay = v2y - v1y;
         vaz = v2z - v1z;
         vbx = v3x - v1x;
         vby = v3y - v1y;
         vbz = v3z - v1z;
	
        nx = (vby * vaz) - (vbz * vay);
	ny = (vbz * vax) - (vbx * vaz);
	nz = (vbx * vay) - (vby * vax);*/

         tmpVec1.x = v2x - v1x;
         tmpVec1.y = v2y - v1y;
         tmpVec1.z = v2z - v1z;
         tmpVec2.x = v3x - v1x;
         tmpVec2.y = v3y - v1y;
         tmpVec2.z = v3z - v1z;
         norm = glm::cross(tmpVec2, tmpVec1);

         nx = norm.x;
         ny = norm.y;
         nz = norm.z;


	l = glm::length(norm);

	if (l != 0) {
	    l = 1 / l;
	    texts[texIndice + 2/*2*/] = normal[indice + 3] = nx * l;
	    texts[texIndice + 3/*3*/] = normal[indice + 4] = ny * l;
	    normal[indice + 5] = nz * l;
        }
	else {
	    texts[texIndice + 2/*2*/] = normal[indice + 3] = 0;
	    texts[texIndice + 3/*3*/] = normal[indice + 4] = 1;
	    normal[indice + 5] = 0;
        }

        //Calculate repeated vertice normals and texture coordinates.
	if (j != 0) {
	    preindice = 6 * (i + (j - 1) * (RESOLUTION + 1));
	    texts[texIndice] = normal[indice] = normal[preindice + 3];
	    texts[texIndice + 1] = normal[indice + 1] = normal[preindice + 4];
	    normal[indice + 2] = normal[preindice + 5];
        }
	else {
 	    //v1x = v1x;
	    v1y = getHeight(v1x, (j - 1) * delta - 1, t);
	    v1z = (j - 1) * delta - 1;

 	   // v3x = v3x;
	    v3y = getHeight(v3x, v2z, t);
	    v3z = v2z;

	    /*vax = v1x - v2x;
	    vay = v1y - v2y;
	    vaz = v1z - v2z;

	    vbx = v3x - v2x;
	    vby = v3y - v2y;
	    vbz = v3z - v2z;

	    nx = (vby * vaz) - (vbz * vay);
	    ny = (vbz * vax) - (vbx * vaz);
	    nz = (vbx * vay) - (vby * vax);*/


            tmpVec1.x = v1x - v2x;
            tmpVec1.y = v1y - v2y;
            tmpVec1.z = v1z - v2z;
            tmpVec2.x = v3x - v2x;
            tmpVec2.y = v3y - v2y;
            tmpVec2.z = v3z - v2z;
            norm = glm::cross(tmpVec2, tmpVec1);

            nx = norm.x;
            ny = norm.y;
            nz = norm.z;

	    l = glm::length(norm);//sqrtf (nx * nx + ny * ny + nz * nz);
	    if (l != 0) {
		l = 1 / l;
		texts[texIndice] = normal[indice] = nx * l;
		texts[texIndice + 1] = normal[indice + 1] = ny * l;
	        normal[indice + 2] = nz * l;
	    }
	    else {
		texts[texIndice] = normal[indice] = 0;
		texts[texIndice + 1] = normal[indice + 1] = 1;
		normal[indice + 2] = 0;
	    }
	  }
      }
   }
   

   SetTexModel();
   SetTexMaterial(3);

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, 1);
    safe_glUniform1i(h_tuTexUnit, 1);

    safe_glEnableVertexAttribArray(h_taTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, WTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(texts), texts, GL_STATIC_DRAW);
    safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 

   safe_glEnableVertexAttribArray(h_taNormal);
   glBindBuffer(GL_ARRAY_BUFFER, waterNormObj);
   glBufferData(GL_ARRAY_BUFFER, sizeof(normal), normal, GL_STATIC_DRAW);
   safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
   
   safe_glEnableVertexAttribArray(h_taPosition);
   glBindBuffer(GL_ARRAY_BUFFER, waterBuffObj);
   glBufferData(GL_ARRAY_BUFFER, sizeof(surface), surface, GL_STATIC_DRAW);
   safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
   	// bind ibo
   	//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
  
  for (i = 1; i < RESOLUTION; i++) {
    glDrawArrays (GL_TRIANGLE_STRIP, i * mylength, mylength);

    /*glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, 1);
    safe_glUniform1i(h_tuTexUnit, 1);*/

    /*safe_glEnableVertexAttribArray(h_taTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, WTexBuffObj);
    safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0);*/
  //printf("%d\n", h_taNormal); 
   
   /*safe_glEnableVertexAttribArray(h_taNormal);
   glBindBuffer(GL_ARRAY_BUFFER, waterNormObj);
   glBufferData(GL_ARRAY_BUFFER, sizeof(normal), normal, GL_STATIC_DRAW);
   safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);*/
  }
  
  safe_glDisableVertexAttribArray(h_taPosition);
  safe_glDisableVertexAttribArray(h_taNormal);
    glDisable(GL_TEXTURE_2D);
}

void DrawIsland(float x, float z, int numTrees) {

    SetTexMaterial(3);
    /**********Draw Land *******************/
    ModelTrans.loadIdentity();
    ModelTrans.pushMatrix();
      ModelTrans.translate(vec3(x, 0, z)); 
      SetTexModel();    
      safe_glEnableVertexAttribArray(h_taPosition);
      // bind vbo
      glBindBuffer(GL_ARRAY_BUFFER, land->PositionHandle);
      safe_glVertexAttribPointer(h_taPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
      // bind ibo
      glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, land->IndexHandle);
      // You will need to also set up the normal buffer like this when your code is ready
      safe_glEnableVertexAttribArray(h_taNormal);
      glBindBuffer(GL_ARRAY_BUFFER, land->NormalHandle);
      safe_glVertexAttribPointer(h_taNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
      
      glEnable(GL_TEXTURE_2D);
      glActiveTexture(GL_TEXTURE3);
      glBindTexture(GL_TEXTURE_2D, 3);
      safe_glUniform1i(h_tuTexUnit, 3);

      safe_glEnableVertexAttribArray(h_taTexCoord);
      glBindBuffer(GL_ARRAY_BUFFER, land->TextureHandle);
      safe_glVertexAttribPointer(h_taTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0); 

      // set the color in the shader 
      glUniform3f(h_uColor, 0.08, 0.78, 0.17);

      // draw!
      //glDrawElements(GL_TRIANGLES, g_LiboLen, GL_UNSIGNED_SHORT, 0);
      glDrawElements(GL_TRIANGLES, land->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
      safe_glDisableVertexAttribArray(h_taPosition);
      safe_glDisableVertexAttribArray(h_taNormal);
      glDisable(GL_TEXTURE_2D);
      if(numTrees == 3) {
       //ModelTrans.loadIdentity();
       SetTexModel();
       
       DrawPalm(vec3(1, 1, -3), 0, vec3(1, 1, 1));
       DrawPalm(vec3(-2, 1, 4), 0, vec3(1, 1, 1));
       DrawPalm(vec3(3, 1, 3), 0, vec3(1, 1, 1));
      }
      else {
       //ModelTrans.loadIdentity();
       SetTexModel();
       
       DrawPalm(vec3(1, 1, -3), 0, vec3(1, 1, 1));
       DrawPalm(vec3(-2, 1, 4), 0, vec3(1, 1, 1));
       //DrawPalm(vec3(3, 1, 3), 0, vec3(1, 1, 1));
      }
    ModelTrans.popMatrix();

}


void checkInputs() {
   float tempx, tempy, tempz;
   if(velocity > 0.09) {
      velocity = 0.09;
   }
   if(velocity < -0.09) {
      velocity = -0.09;
   }
   tempx = eyeVec.x + (velocity) * wVec.x;
   tempz = eyeVec.z + (velocity) * wVec.z;

   if( tempx < WATER_WIDTH_DEPTH - 5 && tempx > -WATER_WIDTH_DEPTH + 5) {
      eyeVec.x = tempx;
      lookAtVec.x += (velocity) * wVec.x;
   }
   if (tempz < WATER_WIDTH_DEPTH - 5 && tempz > -WATER_WIDTH_DEPTH + 5) {
      eyeVec.z = tempz;
      lookAtVec.z += (velocity) * wVec.z;
   }
   
   if(!goforward && !goback) {
      if(velocity < 0) {
         velocity += 0.0005;
         if(velocity > 0) {
            velocity = 0;
         }
      }
      else if (velocity > 0) {
         velocity -= 0.0005;
         if(velocity < 0) {
            velocity = 0;
         }
      }
   }
   if(goleft) {
      beta -= 0.08/M_PI;
      tempx = cos(alpha)*cos(beta);
      tempy = sin(alpha);
      tempz = cos(alpha)*cos(M_PI/2.0 - beta);
      if (alpha < M_PI/2 && alpha > -M_PI/2) {
        lookAtVec.x = tempx + eyeVec.x;
        lookAtVec.y = tempy + eyeVec.y;
        lookAtVec.z = tempz + eyeVec.z;
      }
   }
   else if (goright) {
      beta += 0.08/M_PI;
      tempx = cos(alpha)*cos(beta);
      tempy = sin(alpha);
      tempz = cos(alpha)*cos(M_PI/2.0 - beta);
      if (alpha < M_PI/2 && alpha > -M_PI/2) {
        lookAtVec.x = tempx + eyeVec.x;
        lookAtVec.y = tempy + eyeVec.y;
        lookAtVec.z = tempz + eyeVec.z;
      }
   }
}

/* Main display function */
void Draw (void)
{
   checkInputs();
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);				
    glUseProgram(TexShadeProg);
    ModelTrans.loadIdentity(); 
    SetTexModel();
    //SetView(); 
    //SetProjectionMatrix();
    SetTexProjectionMatrix();
    safe_glUniform3f(h_tuLightPos, 0, 60, 0.0f);
    safe_glUniform3f(h_tuView, -eyeVec.x, -eyeVec.y, -eyeVec.z);
    safe_glUniform3f(h_tuLightColor, 1.0f, 1.0f, 1.0f);
    safe_glUniform1i(h_tuWaterToggle, 0);
    SetTexView(); 
    
    DrawBoat(vec3(eyeVec.x, eyeVec.y -0.1, eyeVec.z), -90 - beta*180/M_PI, vec3(0.1, .1, .1));
    safe_glUniform1i(h_tuWaterToggle, 1);
    DrawSky();   
    drawSea();
    
    //glUseProgram(0);
	
     //Start our shader	
     
     //glUseProgram(ShadeProg);

	/* only set the projection and view matrix once */
    SetTexProjectionMatrix();
    SetTexView(); 
    
    //safe_glUniform3f(h_uLightPos, 0, 60.0f, 0.0f);
    //safe_glUniform3f(h_uView, -eyeVec.x, -eyeVec.y, -eyeVec.z);
    //safe_glUniform3f(h_uLightColor, 1.0f, 1.0f, 1.0f);
    
    /* set the matrix stack to the identity */
     //SetMaterial(3);
     ModelTrans.loadIdentity();
     SetTexModel();
   
   DrawIsland(0, 0, 3);
   DrawIsland(-20, 30, 2);
   DrawIsland(30, 20, 3);

    
    glUseProgram(0);
    glutSwapBuffers();
  //glutPostRedisplay();
}

/* Reshape */
void ReshapeGL (int width, int height)								
{
	g_width = (float)width;
	g_height = (float)height;
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				

}

void keyUp(unsigned char key, int x, int y) {
   switch(key) {
      case 'a':
         goleft = false;
         break;
      case 'd':
         goright = false;
         break;
      case 'w':
         goforward = false;
         break;
      case 's':
         goback = false;
         break;
      default:
         break;
   }
}
//the keyboard callback to change the values to the transforms
void keyboard(unsigned char key, int x, int y ){
  gaze = lookAtVec - eyeVec;
  wVec = glm::normalize(-gaze);
  uVec = glm::cross(upVec, wVec);
  uVec = glm::normalize(uVec);
  vVec = glm::cross(wVec, uVec);
  vVec = glm::normalize(vVec);
  float tempx, tempy, tempz;
  switch( key ) {
    /* WASD keyes effect view/camera transform */
    case 'w':
      //g_trans += 0.1;
      goforward = true;
      velocity -= 0.01; 
      break;
    case 's':
      goback = true;
      velocity += 0.01;
      break;
    case 'a':
      //g_angle += 1;
      goleft = true;
      break;
    case 'd':
      goright = true;
      break;
    /* M and N key move the object in x */
    /*case 'n':
      g_tx -= .1;
      break;
    case 'm':
      g_tx += .1;
      break;
    case 'i':
      Upy++;
      break;
    case 'l':
      Upx++;
      break;
    case 'j':
      Upx--;
      break;
    case 'k':
      Upy--;
      break;
    case 'z':
        rotateFore++;
        rotateUp++;
      break;
    case 'x':
        rotateFore--;
        rotateUp--;
      break;*/
    case 'q': case 'Q' :
      exit( EXIT_SUCCESS );
      break;
  }
  glutPostRedisplay();
}


float p2wx(int in_x) {
  //fill in with the correct return value
  float new_x;
  float L, R, C, D;
  if (g_width > g_height) {
     L = -g_width/g_height;
     R = g_width/g_height;
 }
  else {
     L = -1;
     R = 1;
  }
  C = (-g_width + 1) / (L -R);
  D = -C * L; 
  new_x = (in_x - D)/C;
  return new_x;
}

float p2wy(int in_y) {
  //flip glut y
  in_y = g_height - in_y;
  float B, T, E, F, new_y;
  if (g_width < g_height) {
     B = -g_height/g_width;
     T = g_height/g_width;
  }
  else {
     B = -1;
     T = 1;
  }
  E = (-g_height + 1) / (B -T);
  F = -E * B; 
  new_y = (in_y - F)/E;

  //fill in with the correct return value
  return new_y;
}

int mousePress = 0;
float mx, my;

/* example mouse function  - updates the buffer with new points */
void Mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
       mousePress = 1;
       mx = p2wx(x);
       my = p2wy(y);
    }
    else if (state == GLUT_UP) {
       mousePress = 0;
    }
  }
  glutPostRedisplay();
	  
}
   
bool notZeroVec(glm::detail::tvec3<float> test) {
   if (test.x || test.y || test.z) {
      return true;
   }
   return false;
}

void OnMotion(int x, int y) {
   float tempx, tempy, tempz;
   if (mousePress) {
      beta += (p2wx(x) - mx)/M_PI;
      alpha += (p2wy(y) - my)/M_PI;
      tempx = cos(alpha)*cos(beta);
      tempy = sin(alpha);
      tempz = cos(alpha)*cos(M_PI/2.0 - beta);
      //gaze.x = tempx;
      //gaze.y = tempy;
      //gaze.z = tempz;
      if (alpha < M_PI/2 && alpha > -M_PI/2) {
      //if (notZeroVec(glm::cross(gaze, glm::vec3(0, 1, 0)))) {
        lookAtVec.x = tempx + eyeVec.x;
        lookAtVec.y = tempy + eyeVec.y;
        lookAtVec.z = tempz + eyeVec.z;
      }
      //gaze.x = vx;
      //gaze.y = vy;
      //gaze.z = vz;
      mx = p2wx(x);
      my = p2wy(y);
      glutPostRedisplay();
   }
}

int waterAng = 0;

void updateBoat(int value) {
   //printf("updating!\n");
   waterAng = (waterAng + 2)%(360);
   yPos = cos(waterAng * M_PI/180)/8;
   boyAng = -sin(waterAng * M_PI/180)*10;
   //cout << "yPos: " << yPos << endl;
   //glutPostRedisplay();
   
   glutTimerFunc(10, updateBoat, 0);
   glutPostRedisplay();
}

int main( int argc, char *argv[] )
{
  unsigned char total_texture[4 * 256 * 256];
  unsigned char alpha_texture[256 * 256];
  unsigned char caustic_texture[3 * 256 * 256];
  unsigned int i;
   	glutInit( &argc, argv );
   	glutInitWindowPosition( 20, 20 );
   	glutInitWindowSize( 800, 800 );
   	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   	glutCreateWindow("Boat Simulation!");
   	glutReshapeFunc( ReshapeGL );
   	glutDisplayFunc( Draw );
	glutMouseFunc( Mouse );
        glutMotionFunc( OnMotion );
        glutKeyboardFunc( keyboard );
        glutKeyboardUpFunc(keyUp);
        glutTimerFunc(10, updateBoat, 0);
   	Initialize();
        LoadTexture((char *)"Textures/ash.bmp", 0);
        LoadTexture((char *)"Textures/water2.bmp", 1);
        LoadTexture((char *)"Textures/skydome.bmp", 2);
	LoadTexture((char *)"Textures/sand.bmp", 3);
        LoadTexture((char *)"Textures/palmleaf.bmp", 5);
        LoadTexture((char *)"Textures/palm.bmp", 4);
        //test the openGL version
	getGLversion();
	//install the shader
	if (!InstallShader(textFileRead((char *)"mesh_vert.glsl"), textFileRead((char *)"mesh_frag.glsl"))) {
		printf("Error installing shader!\n");
		return 0;
	}
        /*if (!InstallTexShader(textFileRead((char *)"tex_vert.glsl"), textFileRead((char *)"tex_frag.glsl"))) {
            printf("Error installing shader!\n");
            return 0;
        }*/

        if (!InstallTexShader(textFileRead((char *)"texlight_vert.glsl"), textFileRead((char *)"texlight_frag.glsl"))) {
            printf("Error installing shader!\n");
            return 0;
        }
	
        InitGeom();
  	glutMainLoop();
   	return 0;
}


