
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr
#include "GLSL_helper.h"
#include "MStackHelp.h"
#include "GeometryCreator.h"
#include "CMeshLoaderSimple.h"
#include "TextureLoader.h"
#include "ParticleSystem.h"

//flag and ID to toggle on and off the shader
int shade = 1;
int ShadeProg;
int TexShadeProg;

/* Global data for water updating to speed up calculations.
 * Do not want to recreate variables each time it's run.*/
glm::vec3 tmpVec1(0);
glm::vec3 tmpVec2(0);
glm::vec3 norm(0);

float v1x;
float v1y;
float v1z;

float v2x;
float v2y;
float v2z;

float v3x;
float v3y;
float v3z;

float nx;
float ny;
float nz;

float l;

unsigned int i;
unsigned int j;
float x;
float z;
unsigned int indice;
unsigned int preindice;

//resolution, or how many triangles in a row/column for water mesh.
#define		RESOLUTION 128
//dimensions of the water mesh.
#define  WATER_WIDTH_DEPTH 40

static float	surface[6 * RESOLUTION * (RESOLUTION + 1)];
static float	texts[4 * RESOLUTION * (RESOLUTION + 1)];
static float	normal[6 * RESOLUTION * (RESOLUTION + 1)];
//end of global data for water mesh.


const unsigned int mylength = 2 * (RESOLUTION + 1);
const float delta = 2.0 / RESOLUTION;

//made this value global, it never changes.
const float xn = 20*(RESOLUTION + 1) * delta + 20;

//Meshes
Mesh *boatBase;
Mesh *boatSides;
Mesh *boatStern;
Mesh *boatBow;
Mesh *land;
Mesh *water;
Mesh *personShadow;
Mesh *skyCube;
Mesh *palmBase;
Mesh *palmLeaf;
Mesh *particle;

GLuint triBuffObj, colBuffObj, normalBuffObj;
int TriangleCount;
//Handles to the shader data
GLint h_aPosition;
GLint h_uColor;
GLint h_aNormal;
GLint h_uModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;
GLint h_uLightPos;
GLint h_uLightColor;
GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine, h_Phong, h_uView;

//handles to texture shader data
GLint h_tuTexUnit;
GLint h_tuTexUnit2;
GLint h_taPosition;
GLint h_taNormal;
GLint h_taTexCoord;
GLint h_tuModelMatrix;
GLint h_tuViewMatrix;
GLint h_tuProjMatrix;
GLint h_tuModelMatrix2;
GLint h_tuViewMatrix2;
GLint h_tuProjMatrix2;
GLint h_tuWaterToggle;
GLint h_tuLeaf;

GLint h_tuLightPos;
GLint h_tuLightColor;
GLint h_tuMatAmb, h_tuMatDif, h_tuMatSpec, h_tuMatShine, h_tPhong, h_tuView;

/*Manual buffer objects for objects I created*/
GLuint waterBuffObj, waterNormObj, WTexBuffObj;
GLuint PTexBuffObj, PLTexBuffObj;
GLuint STexBuffObj, SkyNormObj;

/* globals to control positioning and window size */
static float g_width, g_height;

/*global data for boat rotation*/
int rotateFore = 0;
bool rotateFlag = true;
int rotateUp = 0;
bool goforward = false;
bool goback = false;
bool goleft = false;
bool goright = false;
int boyAng;
float yPos = 0;
float velocity = 0;
float elapsedTime = 0;

/*Global camera data*/
float vx = 0.0, vy=1.5, vz=14.0;
glm::vec3 gaze = glm::vec3(vx, vy, vz);
#define _USE_MATH_DEFINES
float alpha = 0.0, beta=-M_PI/2.0;
glm::vec3 wVec = glm::vec3(0);
glm::vec3 uVec = glm::vec3(0);
glm::vec3 vVec = glm::vec3(0);
//glm::vec3 gazeVec = glm::vec3(vx, vy, vz);
glm::vec3 lookAtVec = glm::vec3(vx, vy, vz);
glm::vec3 upVec = glm::vec3(0, 1, 0);
glm::vec3 eyeVec = glm::vec3(0, 1.5, 15);

/*Global particle data*/
ParticleSystem allParticles(-1, NULL);
int material[NUM_PARTICLES];

//declare a matrix stack
RenderingHelper ModelTrans;


static GLuint	texture;
static GLuint TexBuffObj;


